
/**************************************************************************
 *
 *  $Id: pcpsutil.h 1.23 2017/05/10 15:26:09 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for pcpsutil.c.
 *
 * -----------------------------------------------------------------------
 *  $Log: pcpsutil.h $
 *  Revision 1.23  2017/05/10 15:26:09  martin
 *  Tiny cleanup.
 *  Revision 1.22  2017/03/17 11:45:51  martin
 *  Moved binary fraction conversion functions to cfg_hlp.h.
 *  Revision 1.21  2015/10/26 16:09:58  martin
 *  Moved definition of PCPS_SER_PACK to deviohlp.h.
 *  The .c file is now obsolete, so there are no prototypes anymore.
 *  Revision 1.20  2014/07/22 12:22:27Z  martin
 *  Fixed a compiler warning.
 *  Revision 1.19  2014/07/14 15:43:23Z  martin
 *  Updated doxygen comment.
 *  Revision 1.18  2014/06/26 14:52:50  martin
 *  Fixed build on targets not supporting 64 bit types.
 *  Updated some comments to doxygen format.
 *  Revision 1.17  2014/05/27 10:39:33Z  martin
 *  More inline functions to convert binary fractions by thomas-b.
 *  Moved definitions of some signal constants to pcpsdefs.h.
 *  Revision 1.16  2013/03/04 15:51:02Z  martin
 *  Added dfrac_sec_from_bin().
 *  Revision 1.15  2012/10/15 09:41:32  martin
 *  Cleaned up handling of pragma pack().
 *  Revision 1.14  2009/03/09 13:39:45  martin
 *  Made pcps_exp_year() an inline function.
 *  Revision 1.13  2008/12/10 19:59:48  martin
 *  Made frac_sec_from_bin() an inline function.
 *  Revision 1.12  2008/11/25 09:59:01  martin
 *  Moved definitions of PCPS_HRT_FRAC_SCALE and
 *  PCPS_HRT_FRAC_SCALE_FMT to pcpsdefs.h.
 *  Revision 1.11  2006/06/29 10:15:02Z  martin
 *  Updated function prototypes.
 *  Revision 1.10  2005/01/14 10:16:12Z  martin
 *  Updated function prototypes.
 *  Revision 1.9  2004/11/09 14:30:50Z  martin
 *  Redefined interface data types using C99 fixed-size definitions.
 *  Updated function prototypes.
 *  Revision 1.8  2004/04/14 09:22:09Z  martin
 *  Pack structures 1 byte aligned.
 *  Revision 1.7  2001/11/28 14:41:25Z  MARTIN
 *  Changed PCPS_HRT_FRAC_SCALE and PCPS_HRT_FRAC_SCALE_FMT
 *  to print 7 rather than 6 digits.
 *  Revision 1.6  2001/09/14 11:59:33  MARTIN
 *  Support for PCPS_HR_TIME fraction conversion.
 *  Updated function prototypes.
 *  Revision 1.5  2001/08/14 12:06:44  MARTIN
 *  Defined constants used to draw a signal bar
 *  depending on a DCF77 clock's signal value.
 *  Revision 1.4  2001/03/01 14:03:18  MARTIN
 *  Updated function prototypes.
 *  Revision 1.3  2000/08/31 14:06:05  MARTIN
 *  Updated function prototypes.
 *  Revision 1.2  2000/07/21 13:43:40  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#ifndef _PCPSUTIL_H
#define _PCPSUTIL_H


/* Other headers to be included */

#include <pcpsdefs.h>
#include <use_pack.h>


#ifdef _PCPSUTIL
 #define _ext
#else
 #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )
  #pragma pack( 1 )      // set byte alignment
  #define _USING_BYTE_ALIGNMENT
#endif

#ifdef __cplusplus
extern "C" {
#endif


/**
 * @brief Expand a 2-digit year number to a 4-digit year number
 *
 * The resulting year number includes the century and is
 * in the range [year_lim ... ( year_lim + 99 )].
 *
 * @param[in] year      The 2-digit year number to be converted
 * @param[in] year_lim  The smallest 4-digit year number to be returned
 *
 * @return The resulting 4 digit year number including century
 */
static __mbg_inline
uint16_t pcps_exp_year( uint8_t year, uint16_t year_lim )
{
  uint16_t lyear = (uint16_t) ( (uint16_t) year + year_lim
                              - ( year_lim % 100 ) );

  if ( lyear < year_lim )
    lyear += 100;

  return lyear;

}  // pcps_exp_year



/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

/* (no header definitions found) */

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif



#if defined( _USING_BYTE_ALIGNMENT )
  #pragma pack()      // set default alignment
  #undef _USING_BYTE_ALIGNMENT
#endif

/* End of header body */

#undef _ext


#endif  /* _PCPSUTIL_H */
